import typing as t
from blok.errors import ProtocolNotCompliantError
from blok.blok import Blok
import importlib


def check_allowed_module_string(value: str) -> bool:
    for char in value:
        if char not in "abcdefghijklmnopqrstuvwxyz_":
            return False

    return True


def check_protocol_compliance(instance: t.Type, protocol: t.Type) -> bool:
    protocol_methods = {
        name
        for name in dir(protocol)
        if getattr(protocol, name, None) and not name.startswith("_")
    }

    instance_methods = {
        name
        for name in dir(instance)
        if getattr(instance, name, None) and not name.startswith("_")
    }

    missing_methods = protocol_methods - instance_methods

    if missing_methods:
        raise ProtocolNotCompliantError(
            f"Instance of {type(instance).__name__} does not implement "
            + f"the following methods required by {protocol.__name__}"
            + ":\n\t-"
            + "\n\t -".join(missing_methods)
        )

    return True


def lazy_load_blok(path) -> Blok:
    # lazily loading a command, first get the module name and attribute name
    import_path = path

    blok_name = "_".join(import_path.split("."))
    modname, cmd_object_name = import_path.rsplit(".", 1)
    # do the import
    mod = importlib.import_module(modname)
    # get the Command object from that module
    cmd_object = getattr(mod, cmd_object_name)
    # check the result to make debugging easier

    blok = cmd_object()
    check_protocol_compliance(blok, Blok)

    return blok_name, blok


def remove_empty_dicts(d):
    if not isinstance(d, dict):
        return d

    non_empty_items = {}
    for key, value in d.items():
        if isinstance(value, dict):
            cleaned_dict = remove_empty_dicts(value)
            if cleaned_dict:  # Only add non-empty dictionaries
                non_empty_items[key] = cleaned_dict
        else:
            non_empty_items[key] = value

    return non_empty_items


def get_prepended_values(kwargs: t.Dict[str, t.Any], blok_name: str):
    prepended = {
        key.split("_", 1)[1]: value
        for key, value in kwargs.items()
        if key.startswith(blok_name)
    }

    return prepended



import os
from pathlib import Path
from blok.tree.models import YamlFile, Repo
import yaml


def create_structure_from_files_and_folders(base_path, omit: list[str] | None = None):
    if omit is None:
        omit = []
    structure = {}

    for root, dirs, files in os.walk(base_path):
        rel_path = Path(root).relative_to(base_path)
        current_dict = structure

        if "__repo__.txt" in files:
            with (Path(root) / "__repo__.txt").open("r") as repo_file:
                repo_url = repo_file.read().strip()

            for part in rel_path.parts[:-1]:
                current_dict = current_dict.setdefault(part, {})
            current_dict[rel_path.parts[-1]] = Repo(repo_url)

        else:
            for part in rel_path.parts:
                current_dict = current_dict.setdefault(part, {})

        for dir_name in dirs:
            current_dict[dir_name] = {}

        for file_name in files:
            if omit and file_name in omit:
                continue

            if file_name == "__repo__.txt":
                continue

            file_path = Path(root) / file_name

            try:
                with file_path.open("rb") as file:
                    content = file.read()

                if file_name.endswith(".yaml") or file_name.endswith(".yml"):
                    content_decoded = content.decode()
                    yaml_content = yaml.safe_load(content_decoded)
                    current_dict[file_name] = YamlFile(**yaml_content)
                else:
                    try:
                        content_decoded = content.decode()
                        current_dict[file_name] = content_decoded
                    except UnicodeDecodeError:
                        current_dict[file_name] = content
            except Exception as e:
                print(f"Failed to read {file_path}")

    return structure
